(**************************************************************************)
(*                                   Ott                                  *)
(*                                                                        *)
(*        Peter Sewell, Computer Laboratory, University of Cambridge      *)
(*      Francesco Zappa Nardelli, Moscova project, INRIA Rocquencourt     *)
(*                                                                        *)
(*  Copyright 2005-2010                                                   *)
(*                                                                        *)
(*  Redistribution and use in source and binary forms, with or without    *)
(*  modification, are permitted provided that the following conditions    *)
(*  are met:                                                              *)
(*  1. Redistributions of source code must retain the above copyright     *)
(*  notice, this list of conditions and the following disclaimer.         *)
(*  2. Redistributions in binary form must reproduce the above copyright  *)
(*  notice, this list of conditions and the following disclaimer in the   *)
(*  documentation and/or other materials provided with the distribution.  *)
(*  3. The names of the authors may not be used to endorse or promote     *)
(*  products derived from this software without specific prior written    *)
(*  permission.                                                           *)
(*                                                                        *)
(*  THIS SOFTWARE IS PROVIDED BY THE AUTHORS ``AS IS'' AND ANY EXPRESS    *)
(*  OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED     *)
(*  WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE    *)
(*  ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY       *)
(*  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL    *)
(*  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE     *)
(*  GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS         *)
(*  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER  *)
(*  IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR       *)
(*  OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN   *)
(*  IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.                         *)
(**************************************************************************)

open Types;;

let rec pp_embeds m xd lookup el =
  String.concat "" (List.map (pp_embedmorphism m xd lookup) el)

and pp_embedmorphism m xd lookup (l,hn,es) =
  match (m,hn) with 
  | (Ascii ao, _) -> 
      Grammar_pp.pp_DOUBLELEFTBRACE ^ " "
      ^ (Grammar_pp.pp_hom_name m xd hn) ^ " "
      ^ (pp_embed_spec m xd lookup es) ^ " "
      ^ Grammar_pp.pp_DOUBLERIGHTBRACE
  | (Coq _, "coq") 
  | (Isa _, "isa")                       
  | (Hol _, "hol") 
  | (Twf _, "twf") 
  | (Tex _, "tex") 
  | (Caml _, "ocaml")
    -> (pp_embed_spec m xd lookup es)^"\n" 
  | (Coq _, _) | (Isa _, _) | (Hol _,_) | (Twf _,_) | (Tex _,_) | (Caml _,_) -> ""

and pp_embed_spec m xd lookup es = 
  (* print_string ("pp_embed_spec list length = "^string_of_int (List.length es)^"\n");flush stdout; *)
  String.concat "" (List.map (pp_embed_spec_el m xd lookup) es)

and pp_embed_spec_el m xd lookup ese = 
  match m with 
  | Ascii ao -> 
      ( match ese with
      | Embed_string (l,s) ->  s
      | Embed_inner (l,s) -> Grammar_pp.pp_DOUBLELEFTBRACKET^s^Grammar_pp.pp_DOUBLERIGHTBRACKET )
  | Tex xo when (match ese with Embed_inner (_,"TEX_NAME_PREFIX")->true | _->false) -> 
      xo.ppt_name_prefix
  | Tex _ | Coq _ | Isa _ | Hol _ | Twf _ | Caml _ -> 
      ( match ese with
      | Embed_string (l,s) ->  s

      | Embed_inner (l,s) -> (* "<<<<<<"^ s^">>>>>" )  *)
          (* not taking account of possible dot forms shared over different terms *)
          let st = Term_parser.just_one_parse m xd lookup "user_syntax" false l s in
          let ((de1,de2) as de,de3,pptbe) = Bounds.bound_extraction m xd l [st]  in
          Grammar_pp.pp_symterm m xd [] de st)
